@@ -22,29 +22,53 @@
 #include <sys/ioctl.h>
 #include <sys/param.h>
 #include <sys/sysmacros.h>
+#include <sys/stat.h>
+#include <inttypes.h>
 #include <malloc.h>
 #include <getopt.h>
+#include <errno.h>
 #include <endian.h>
+#include <stdbool.h>
+#include <asm/types.h>
+#include <mntent.h>
 
 static __inline__ int xfsctl(const char *path, int fd, int cmd, void *p)
 {
 	return ioctl(fd, cmd, p);
 }
 
+/*
+ * platform_test_xfs_*() implies that xfsctl will succeed on the file;
+ * on Linux, at least, special files don't get xfs file ops,
+ * so return 0 for those
+ */
+
 static __inline__ int platform_test_xfs_fd(int fd)
 {
-	struct statfs buf;
-	if (fstatfs(fd, &buf) < 0)
+	struct statfs statfsbuf;
+	struct stat statbuf;
+
+	if (fstatfs(fd, &statfsbuf) < 0)
 		return 0;
-	return (buf.f_type == 0x58465342);	/* XFSB */
+	if (fstat(fd, &statbuf) < 0)
+		return 0;
+	if (!S_ISREG(statbuf.st_mode) && !S_ISDIR(statbuf.st_mode))
+		return 0;
+	return (statfsbuf.f_type == 0x58465342);	/* XFSB */
 }
 
 static __inline__ int platform_test_xfs_path(const char *path)
 {
-	struct statfs buf;
-	if (statfs(path, &buf) < 0)
+	struct statfs statfsbuf;
+	struct stat statbuf;
+
+	if (statfs(path, &statfsbuf) < 0)
 		return 0;
-	return (buf.f_type == 0x58465342);	/* XFSB */
+	if (stat(path, &statbuf) < 0)
+		return 0;
+	if (!S_ISREG(statbuf.st_mode) && !S_ISDIR(statbuf.st_mode))
+		return 0;
+	return (statfsbuf.f_type == 0x58465342);	/* XFSB */
 }
 
 static __inline__ int platform_fstatfs(int fd, struct statfs *buf)
@@ -115,21 +139,38 @@ platform_discard_blocks(int fd, uint64_t start, uint64_t len)
 
 #define ENOATTR		ENODATA	/* Attribute not found */
 #define EFSCORRUPTED	EUCLEAN	/* Filesystem is corrupted */
+#define EFSBADCRC	EBADMSG	/* Bad CRC detected */
 
 typedef loff_t		xfs_off_t;
 typedef __uint64_t	xfs_ino_t;
 typedef __uint32_t	xfs_dev_t;
 typedef __int64_t	xfs_daddr_t;
-typedef char*		xfs_caddr_t;
 
-#ifndef	_UCHAR_T_DEFINED
-typedef unsigned char	uchar_t;
-#define	_UCHAR_T_DEFINED	1
-#endif
+/**
+ * Abstraction of mountpoints.
+ */
+struct mntent_cursor {
+	FILE *mtabp;
+};
 
-#ifndef _BOOLEAN_T_DEFINED
-typedef enum {B_FALSE, B_TRUE}	boolean_t;
-#define _BOOLEAN_T_DEFINED	1
-#endif
+static inline int platform_mntent_open(struct mntent_cursor * cursor, char *mtab)
+{
+	cursor->mtabp = setmntent(mtab, "r");
+	if (!cursor->mtabp) {
+		fprintf(stderr, "Error: cannot read %s\n", mtab);
+		return 1;
+	}
+	return 0;
+}
+
+static inline struct mntent * platform_mntent_next(struct mntent_cursor * cursor)
+{
+	return getmntent(cursor->mtabp);
+}
+
+static inline void platform_mntent_close(struct mntent_cursor * cursor)
+{
+	endmntent(cursor->mtabp);
+}
 
 #endif	/* __XFS_LINUX_H__ */
